package com.tin.tds.lower.util;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import com.tin.tds.lower.bean.TDS194PDetailBean;
import com.tin.tds.lower.FieldValidation;
import com.tin.tds.lower.FileValidatorInterface;
import com.tin.tds.lower.FormValidator;
import com.tin.tds.lower.bean.RAWFileStatisticData;
import com.tin.tds.lower.bean.SAMBatchData;
import com.tin.tds.lower.bean.SAMSCMDataBean;
import com.tin.tds.lower.bean.Section16DetailBean;
import com.tin.tds.lower.bean.SectionVIADetailBean;
import com.tin.tds.lower.bean.TDSBatchHeaderBean;
import com.tin.tds.lower.bean.TDSChallanDetailBean;
import com.tin.tds.lower.bean.TDSDeducteeDetailBean;
import com.tin.tds.lower.bean.TDSFileHeaderBean;
import com.tin.tds.lower.bean.TDSSalaryDetailBean;

public class RAWFileGenerator {

	
	public static String generateRawFile(String inputFile)
	{
		int lineCount = 0;
		int noOfBatches=0;
		String fileType = null;
		String uploadType=null;
		SAMBatchData [] batchData = null;
		RAWFileStatisticData [] statisticData = null;
		SAMSCMDataBean [] samscmData = null;
		String [] transactionType = null;
		
		String formNo=null;
		
		try {
			BufferedReader fileReader = new BufferedReader(new FileReader(inputFile));
			
			String lineData = null;
			int currentBatchInProcessing = 0;
			
			while((lineData = fileReader.readLine())!=null)
			{
				lineCount++;
				String [] recordData  = (lineData+"-").split("\\^");
				
				if(recordData[1].equals(FileValidatorInterface.FILE_HEADER_IDENTIFIER))
				{
					TDSFileHeaderBean fhData  = getFileHeaderBean(recordData);
					noOfBatches = Integer.parseInt(fhData.getNoOfBatches(),10);
					fileType = fhData.getFileType();
					uploadType = fhData.getUploadType();
					batchData = new SAMBatchData[noOfBatches];
					statisticData = new RAWFileStatisticData[noOfBatches];
					samscmData = new SAMSCMDataBean[noOfBatches];
					transactionType=new String[noOfBatches]; 
					
				}
				else if(recordData[1].equals(FileValidatorInterface.BATCH_HEADER_IDENTIFIER))
				{
					
					
					
					currentBatchInProcessing++;
					
					TDSBatchHeaderBean bhData = getBatchHeaderBean(recordData);
					formNo = bhData.getFormNo();
					
					if(bhData.getTransactionType() != null) //Added on 01 Mar 2022 FVU 7.5 - Vinita //23 March 2022 added for 7.5_2.171_2.1303
						transactionType[currentBatchInProcessing-1] = new String(bhData.getTransactionType()); 
					
					//transactionType[currentBatchInProcessing-1] = new String(bhData.getTransactionType()); 
					batchData[currentBatchInProcessing-1] = new SAMBatchData();
					statisticData[currentBatchInProcessing-1] = new RAWFileStatisticData();
					samscmData[currentBatchInProcessing-1]= new SAMSCMDataBean();
					
					
					samscmData[currentBatchInProcessing-1].setOrgRRRNo(bhData.getOriginalRrrNo());
					samscmData[currentBatchInProcessing-1].setPrevRRRNo(bhData.getPreviousRrrNo());
					samscmData[currentBatchInProcessing-1].setEmpAdd1(bhData.getEmployerAddr1());
					samscmData[currentBatchInProcessing-1].setEmpAdd2(bhData.getEmployerAddr2());
					samscmData[currentBatchInProcessing-1].setEmpAdd3(bhData.getEmployerAddr3());
					samscmData[currentBatchInProcessing-1].setEmpAdd4(bhData.getEmployerAddr4());
					samscmData[currentBatchInProcessing-1].setEmpAdd5(bhData.getEmployerAddr5());
					samscmData[currentBatchInProcessing-1].setEmpState(bhData.getEmployerState());
					samscmData[currentBatchInProcessing-1].setEmpPin(bhData.getEmployerPin());
					samscmData[currentBatchInProcessing-1].setDedType(bhData.getDeductorType());
					samscmData[currentBatchInProcessing-1].setApprovalTaken(bhData.getApprovalTaken());
					samscmData[currentBatchInProcessing-1].setApprovalNo(bhData.getApprovalNo());
					
					
					statisticData[currentBatchInProcessing-1].setCountOfChallanRec(bhData.getChallanCount());
					statisticData[currentBatchInProcessing-1].setFormNo(bhData.getFormNo());
					statisticData[currentBatchInProcessing-1].setTransactionType(bhData.getTransactionType());
					statisticData[currentBatchInProcessing-1].setTan(bhData.getTanOfDeductor());
					statisticData[currentBatchInProcessing-1].setLastTan(bhData.getLastTanOfDeductor());
					statisticData[currentBatchInProcessing-1].setPan(bhData.getPanOfDeductor());
					statisticData[currentBatchInProcessing-1].setAssYear(bhData.getAssessmentYr());
					statisticData[currentBatchInProcessing-1].setFinYear(bhData.getFinancialYr());
					statisticData[currentBatchInProcessing-1].setPeriod(bhData.getPeriod());
					statisticData[currentBatchInProcessing-1].setEmployerName(bhData.getEmployerName());
					statisticData[currentBatchInProcessing-1].setGrossTDSTotalAsPerChallan(bhData.getGrossTdsTotalAsPerChallan());
					statisticData[currentBatchInProcessing-1].setCountOfSalaryDetailRec(bhData.getCountOfSalaryDetailRec());
					statisticData[currentBatchInProcessing-1].setGrossTotalIncomeSD(bhData.getGrossTotalIncomeSd());
					
					
					batchData[currentBatchInProcessing-1].setBatchNo(String.valueOf(currentBatchInProcessing));
					batchData[currentBatchInProcessing-1].setLastTan(bhData.getLastTanOfDeductor());
					batchData[currentBatchInProcessing-1].setTotalSDRecords((bhData.getCountOfSalaryDetailRec()));
					batchData[currentBatchInProcessing-1].setUnmatchChallanCount(bhData.getUnMatchedChalanCnt());
					//FVU 7.6 BUG FIX new requirement .fvu
					batchData[currentBatchInProcessing-1].setTotal94PRecords((bhData.getCountOf194PDetailRec()));
					
				}
				else if(recordData[1].equals(FileValidatorInterface.CHALLAN_HEADER_IDENTIFIER))
				{
					TDSChallanDetailBean cdData = getChallanDetailBean(recordData);
					
					
					batchData[currentBatchInProcessing-1].addTotalDDRecords(cdData.getCountOfDeducteeDetail());
					statisticData[currentBatchInProcessing-1].addStatTotalNoOfDDRec(cdData.getCountOfDeducteeDetail());
					
					if(!cdData.getTransferVoucherNo().equals("")|| cdData.getBookCash().equals("Y"))
					{
						statisticData[currentBatchInProcessing-1].addStatTransferVoucherCount();
						
						statisticData[currentBatchInProcessing-1].addTransferAmountAsPerTV(cdData.getTotalOfDepositAmountAsPerChallan());
					}
					else if (cdData.getBookCash().equals("N") || !cdData.getBankChallanNo().equals("") || cdData.getNillChallanIndicator().equals("N") || cdData.getNillChallanIndicator().equals("Y"))
					{
						statisticData[currentBatchInProcessing-1].addStatChallanCount();
						
						statisticData[currentBatchInProcessing-1].addChallanAmountAsPerChallan(cdData.getTotalOfDepositAmountAsPerChallan());
					}
				
					
					int countOfChallanRecordsInBatch = Integer.parseInt(statisticData[currentBatchInProcessing-1].getCountOfChallanRec());
					int countOfChallanRecords = Integer.parseInt(statisticData[currentBatchInProcessing-1].getStatChallanCount());
					int countOfTVRecords = Integer.parseInt(statisticData[currentBatchInProcessing-1].getStatTransferVoucherCount());
					
					if(countOfChallanRecordsInBatch == countOfChallanRecords)
					{
						statisticData[currentBatchInProcessing-1].setTypeOfChallan("C");
					}
					else if(countOfChallanRecordsInBatch == countOfTVRecords)
					{
						statisticData[currentBatchInProcessing-1].setTypeOfChallan("T");
					}
					else if(countOfChallanRecordsInBatch == (countOfChallanRecords+countOfTVRecords))
					{
						statisticData[currentBatchInProcessing-1].setTypeOfChallan("B");
					}
					
					
					samscmData[currentBatchInProcessing-1].addTotalOfLastTotalOfDepositAmountAsPerChallan(cdData.getLastTotalOfDepositAmountAsPerChallan());
					
					
					
				}
				else if(recordData[1].equals(FileValidatorInterface.DEDUCTEE_HEADER_IDENTIFIER))
				{
					TDSDeducteeDetailBean ddData = getDeducteeDetailBean(recordData);
					
					
					FieldValidation fieldVal = new FieldValidation();
					if(fieldVal.checkPan(ddData.getEmployeePan())==false)
					{
						statisticData[currentBatchInProcessing-1].addReceiptValidPAN();
					}
					
					
					
					
					if(ddData.getRemark1().equals("C") && !formNo.equals(FormValidator.FORM_27EQ))
					{
						batchData[currentBatchInProcessing-1].addDedAtHigherRate();
						statisticData[currentBatchInProcessing-1].addStatCntTaxDeductedAtHighRate();
					}
					
					if(ddData.getRemark1()!=null && !ddData.getRemark1().equals(""))
					{
						if(!transactionType[currentBatchInProcessing-1].equals(FormValidator.TRANSACTIONTYPE_CORRECTION_BATCH) && !transactionType[currentBatchInProcessing-1].equals(FormValidator.TRANSACTIONTYPE_CORRECTION_CHALLAN) && !transactionType[currentBatchInProcessing-1].equals(FormValidator.TRANSACTIONTYPE_CORRECTION_SALARY) && !formNo.equals(FormValidator.FORM_27EQ))
						{
							batchData[currentBatchInProcessing-1].setDedAtHigherRate(batchData[currentBatchInProcessing-1].getDedAtHigherRate());
						}
					}
					batchData[currentBatchInProcessing-1].addTotalValueOfPurchase(ddData.getTotalValueofPurchase());
					
					
					
					
					
					if(ddData.getEmployeePan().equals(FormValidator.PAN_APPLIED))
					{
						batchData[currentBatchInProcessing-1].addCountOfPanApplied();
					}
					else if(ddData.getEmployeePan().equals(FormValidator.PAN_NOT_AVAILABLE))
					{
						batchData[currentBatchInProcessing-1].addCountOfPanNotAvbl();
					}
					else if(ddData.getEmployeePan().equals(FormValidator.PAN_INVALID))
					{
						batchData[currentBatchInProcessing-1].addCountOfPanInvalid();
					}
					
					
					
					
					if(ddData.getMode().equals("O"))
					{
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountPaidCredited(ddData.getAmountOfPayment());
						batchData[currentBatchInProcessing-1].addDdAmtOriginal(ddData.getAmountOfPayment());
						
						
						batchData[currentBatchInProcessing-1].addTotIcnTaxDedOrg(ddData.getTotalIncomeTaxDeductedAtSource());
						samscmData[currentBatchInProcessing-1].addAdditionTotalTaxDeducted(ddData.getTotalIncomeTaxDeductedAtSource());
						
						batchData[currentBatchInProcessing-1].addTotTaxDepOrg(ddData.getTotalTaxDeposited());
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountDeposited(ddData.getTotalTaxDeposited());
					}
					else if(ddData.getMode().equals("U"))
					{
						
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountPaidCredited(ddData.getAmountOfPayment());
						
						
						batchData[currentBatchInProcessing-1].addTotIcnTaxDedUpd(ddData.getTotalIncomeTaxDeductedAtSource());
						samscmData[currentBatchInProcessing-1].addAdditionTotalTaxDeducted(ddData.getTotalIncomeTaxDeductedAtSource());
						
						
						batchData[currentBatchInProcessing-1].addTotTaxDepUpd(ddData.getTotalTaxDeposited());
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountDeposited(ddData.getTotalTaxDeposited());
						
					}
					else if(ddData.getMode().equals("A"))
					{
						
						batchData[currentBatchInProcessing-1].addDdAmtAdded(ddData.getAmountOfPayment());
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountPaidCredited(ddData.getAmountOfPayment());
						
						
						batchData[currentBatchInProcessing-1].addTotIcnTaxDedAdd(ddData.getTotalIncomeTaxDeductedAtSource());
						samscmData[currentBatchInProcessing-1].addAdditionTotalTaxDeducted(ddData.getTotalIncomeTaxDeductedAtSource());
						
						
						batchData[currentBatchInProcessing-1].addTotTaxDepAdd(ddData.getTotalTaxDeposited());
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountDeposited(ddData.getTotalTaxDeposited());
					}
					else if(ddData.getMode().equals("D"))
					{
						batchData[currentBatchInProcessing-1].addTotIcnTaxDedDel(ddData.getTotalIncomeTaxDeductedAtSource());
						samscmData[currentBatchInProcessing-1].addAdditionTotalTaxDeducted(ddData.getTotalIncomeTaxDeductedAtSource());
						
						
						batchData[currentBatchInProcessing-1].addTotTaxDepDel(ddData.getTotalTaxDeposited());
						samscmData[currentBatchInProcessing-1].addAdditionTotalAmountDeposited(ddData.getTotalTaxDeposited());
					}
					
					
					samscmData[currentBatchInProcessing-1].setSamScmLastTotalIncomeTaxDeductedAtSource(ddData.getLastTotalIncomeTaxDeductedAtSource());
					
					
				}
				else if(recordData[1].equals(FileValidatorInterface.SALARY_STATEMENT_IDENTIFIER))
				{
					TDSSalaryDetailBean salData  = getSalaryDetailBean(recordData);
					batchData[currentBatchInProcessing-1].addTotTaxableIncome340344(salData.getTotTaxableIncome340344());
					
				}
				else if(recordData[1].equals(FileValidatorInterface.SECTION16_HEADER_IDENTIFIER))
				{
					//Section16DetailBean sec16bean = getSection16DetailBean(recordData);
				}
				else if(recordData[1].equals(FileValidatorInterface.SECTIONVIA_HEADER_IDENTIFIER))
				{
					//SectionVIADetailBean secVIAbean = getSectionVIADetailBean(recordData);
				}
				//FVU 7.6 BUG FIX new requirement .fvu
				else if(recordData[1].equals(FileValidatorInterface.SEC_194P_STATEMENT_IDENTIFIER))
				{
					TDS194PDetailBean ninetyFourPData  = get194PDetailBean(recordData);
					batchData[currentBatchInProcessing-1].addTotNetTaxPayable(ninetyFourPData.getNetTaxPayable());
					
				}
			}
			
			fileReader.close();
			
			return getFinalRawFileData(noOfBatches,lineCount,batchData,fileType,uploadType,statisticData,samscmData,transactionType);
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	
	
	
	private static String getFinalRawFileData(int noOfBatches, int lineCount, SAMBatchData[] batchData, String fileType, String uploadType, RAWFileStatisticData[] statisticData, SAMSCMDataBean[] samscmData, String[] transactionType)
	{
		String out = noOfBatches+"^"+lineCount+"^"+Parameters.fvuVersion[0]+"^"+"00000000000000000000"+"^"+Parameters.samVersion[0]+"^"+"00000000000000000000"+"^"+System.getProperty("line.separator");
		
		for(int i=0;i<batchData.length;i++)
		{
			String lineData = "BH="+batchData[i].getBatchNo()+"^P1="+batchData[i].getCountOfPanApplied()+"^P2="+batchData[i].getCountOfPanInvalid()+"^P3="+batchData[i].getCountOfPanNotAvbl()+"^T="+batchData[i].getLastTan()+"^AO="+batchData[i].getDdAmtOriginal()+"^AU="+batchData[i].getDdAmtUpdated()+"^AA="+batchData[i].getDdAmtAdded()+"^DO="+batchData[i].getTotIcnTaxDedOrg()+"^DU="+batchData[i].getTotIcnTaxDedUpd()+"^DA="+batchData[i].getTotIcnTaxDedAdd()+"^DD="+batchData[i].getTotIcnTaxDedDel()+"^TO="+batchData[i].getTotTaxDepOrg()+"^TU="+batchData[i].getTotTaxDepUpd()+"^TA="+batchData[i].getTotTaxDepAdd()+"^TD="+batchData[i].getTotTaxDepDel()+"^VP="+batchData[i].getTotalValueOfPurchase()+"^DD="+batchData[i].getTotalDDRecords()+"^SD="+batchData[i].getTotalSDRecords()+"^DH="+batchData[i].getDedAtHigherRate()+"^UC="+batchData[i].getUnmatchChallanCount()+"^NT="+batchData[i].getTotTaxableIncome340344()+"^94P="+batchData[i].getTotal94PRecords()+"^NP="+batchData[i].getTotNetTaxPayable()+"^~";   //FVU 7.6 BUG FIX new requirement .fvu
			
			out+=lineData;
		}
		
		out+=System.getProperty("line.separator")+fileType+"^"+uploadType+System.getProperty("line.separator");
		
		for(int i=0;i<statisticData.length;i++)
		{
			String lineData = statisticData[i].getCountOfChallanRec()+"^"+statisticData[i].getFormNo()+"^"+statisticData[i].getTransactionType()+"^"+statisticData[i].getTan()+"^"+statisticData[i].getLastTan()+"^"+statisticData[i].getPan()+"^"+statisticData[i].getAssYear()+"^"+statisticData[i].getFinYear()+"^"+statisticData[i].getPeriod()+"^"+statisticData[i].getEmployerName()+"^"+statisticData[i].getGrossTDSTotalAsPerChallan()+"^"+statisticData[i].getCountOfSalaryDetailRec()+"^"+statisticData[i].getGrossTotalIncomeSD()+"^"+((!transactionType[i].equals(FormValidator.TRANSACTIONTYPE_CORRECTION_BATCH))?(statisticData[i].getStatChallanCount()+"^"+statisticData[i].getChallanAmountAsPerChallan()+"^"+statisticData[i].getStatTransferVoucherCount()+"^"+statisticData[i].getTransferAmountAsPerTV()+"^"+(statisticData[i].getTypeOfChallan())+"^"+statisticData[i].getMatchChallanCount()+"^"+statisticData[i].getTotalChallanforCINCheck()+"^"+statisticData[i].getTotalChallanforCINCheck2()+"^"):"")+statisticData[i].getStatTotalNoOfDDRec()+"^"+statisticData[i].getReceiptValidPAN()+"^"+statisticData[i].getStatCntTaxDeductedAtHighRate()+"^"+System.getProperty("line.separator");
			
			out+=lineData;
		}
		
		for(int i=0;i<samscmData.length;i++)
		{
			String lineData = samscmData[i].getOrgRRRNo()+"^"+samscmData[i].getPrevRRRNo()+"^"+samscmData[i].getEmpAdd1()+"^"+samscmData[i].getEmpAdd2()+"^"+samscmData[i].getEmpAdd3()+"^"+samscmData[i].getEmpAdd4()+"^"+samscmData[i].getEmpAdd5()+"^"+samscmData[i].getEmpState()+"^"+samscmData[i].getEmpPin()+"^"+samscmData[i].getDedType()+"^"+samscmData[i].getApprovalTaken()+"^"+samscmData[i].getApprovalNo()+"^"+samscmData[i].getAdditionTotalAmountPaidCredited()+"^"+samscmData[i].getAdditionTotalTaxDeducted()+"^"+samscmData[i].getAdditionTotalAmountDeposited()+"^"+samscmData[i].getTotalOfLastTotalOfDepositAmountAsPerChallan()+"^"+samscmData[i].getSamScmLastTotalIncomeTaxDeductedAtSource()+"^"+samscmData[i].getSamScmRrrNum()+"^"+samscmData[i].getSamScmRrrDate()+"^"+System.getProperty("line.separator");
			
			out+=lineData;
		}
		
		
		return out;
	}
	
	
	private static TDSFileHeaderBean getFileHeaderBean(String [] fhFields)
	{
		
		TDSFileHeaderBean fhRecord = new TDSFileHeaderBean();
		if(fhFields.length==18) //Above 2.126
		{
			fhRecord.setLineNo(fhFields[0]);
			fhRecord.setRecordType(fhFields[1]);
			fhRecord.setFileType(fhFields[2]);
			fhRecord.setUploadType(fhFields[3]);
			fhRecord.setFileDate(fhFields[4]);
			fhRecord.setFileSeq(fhFields[5]);
			fhRecord.setUploaderType(fhFields[6]);
			fhRecord.setTanTfcId(fhFields[7]);
			fhRecord.setNoOfBatches(fhFields[8]);
			fhRecord.setRpuName(fhFields[9]);
			fhRecord.setRecordHash(fhFields[10]);
			fhRecord.setFvuVersion(fhFields[11]);
			fhRecord.setFileHash(fhFields[12]);
			fhRecord.setSamVersion(fhFields[13]);
			fhRecord.setSamHash(fhFields[14]);
			fhRecord.setScmVersion(fhFields[15]);
			fhRecord.setScmHash(fhFields[16]);
			fhRecord.setConsHash(fhFields[17].substring(0, fhFields[17].length()-1));//Removing -
		}
		else if(fhFields.length==16)//2.126 and below
		{
			fhRecord.setLineNo(fhFields[0]);
			fhRecord.setRecordType(fhFields[1]);
			fhRecord.setFileType(fhFields[2]);
			fhRecord.setUploadType(fhFields[3]);
			fhRecord.setFileDate(fhFields[4]);
			fhRecord.setFileSeq(fhFields[5]);
			fhRecord.setUploaderType(fhFields[6]);
			fhRecord.setTanTfcId(fhFields[7]);
			fhRecord.setNoOfBatches(fhFields[8]);
			fhRecord.setRecordHash(fhFields[9]);
			fhRecord.setFvuVersion(fhFields[10]);
			fhRecord.setFileHash(fhFields[11]);
			fhRecord.setSamVersion(fhFields[12]);
			fhRecord.setSamHash(fhFields[13]);
			fhRecord.setScmVersion(fhFields[14]);
			fhRecord.setScmHash(fhFields[15].substring(0, fhFields[16].length()-1)); //Removing -
		}
		
		
		return fhRecord;
	}
	
	private static TDSBatchHeaderBean getBatchHeaderBean(String [] bhFields)
	{
		TDSBatchHeaderBean batchRecords = new TDSBatchHeaderBean();
		
		if(bhFields.length==72)  //Added on 01 Mar 2022 FVU 7.5 - Vinita //23 March 2022 added for 7.5_2.171_2.1303
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			batchRecords.setLastDeductorType(bhFields[52]);
			batchRecords.setStateName(bhFields[53]);
			batchRecords.setPaoCode(bhFields[54]);
			batchRecords.setDdoCode(bhFields[55]);
			batchRecords.setMinistryName(bhFields[56]);
			batchRecords.setMinistryNameOther(bhFields[57]);
			batchRecords.setRegistrationId(bhFields[58]);
			batchRecords.setPaoRegistrationNo(bhFields[59]);
			batchRecords.setDdoRegistrationNo(bhFields[60]);
			batchRecords.setBatchHash(bhFields[61]);
			//FVU 7.6 BUG FIX new requirement .fvu
			batchRecords.setCountOf194PDetailRec(bhFields[69]);
			batchRecords.setGrossTotalIncome94P(bhFields[70]);
		}
		//if(bhFields.length==69) //2.126 above
		else if(bhFields.length==70) //2.152 & abv
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			batchRecords.setLastDeductorType(bhFields[52]);
			batchRecords.setStateName(bhFields[53]);
			batchRecords.setPaoCode(bhFields[54]);
			batchRecords.setDdoCode(bhFields[55]);
			batchRecords.setMinistryName(bhFields[56]);
			batchRecords.setMinistryNameOther(bhFields[57]);
			batchRecords.setRegistrationId(bhFields[58]);
			batchRecords.setPaoRegistrationNo(bhFields[59]);
			batchRecords.setDdoRegistrationNo(bhFields[60]);
			
			//Added for FVU 2.134 by Shalin
			
			batchRecords.setEmployerSTDAlt(bhFields[61]);
			batchRecords.setEmployerPhoneAlt(bhFields[62]);
			batchRecords.setEmployerEmailAlt(bhFields[63]);
			batchRecords.setPersonResponsilbleSTDCodeAlt(bhFields[64]);
			batchRecords.setPersonResponsilbleTelePhoneAlt(bhFields[65]);
			batchRecords.setPersonResponsilbleEmailIdAlt(bhFields[66]);
			batchRecords.setAIN(bhFields[67]);
					
			
			batchRecords.setBatchHash(bhFields[68]);
		}
		else if(bhFields.length==53)
		{
			batchRecords.setLineNo(bhFields[0]);
			batchRecords.setRecordType(bhFields[1]);
			batchRecords.setBatchNo(bhFields[2]);
			batchRecords.setChallanCount(bhFields[3]);
			batchRecords.setFormNo(bhFields[4]);
			batchRecords.setTransactionType(bhFields[5]);
			batchRecords.setBatchUpdationIndicator(bhFields[6]);
			batchRecords.setOriginalRrrNo(bhFields[7]);
			batchRecords.setPreviousRrrNo(bhFields[8]);
			batchRecords.setRrrNo(bhFields[9]);
			batchRecords.setRrrDate(bhFields[10]);
			batchRecords.setLastTanOfDeductor(bhFields[11]);
			batchRecords.setTanOfDeductor(bhFields[12]);
			batchRecords.setReceiptNumber(bhFields[13]);
			batchRecords.setPanOfDeductor(bhFields[14]);
			batchRecords.setAssessmentYr(bhFields[15]);
			batchRecords.setFinancialYr(bhFields[16]);
			batchRecords.setPeriod(bhFields[17]);
			batchRecords.setEmployerName(bhFields[18]);
			batchRecords.setEmployerBranchDiv(bhFields[19]);
			batchRecords.setEmployerAddr1(bhFields[20]);
			batchRecords.setEmployerAddr2(bhFields[21]);
			batchRecords.setEmployerAddr3(bhFields[22]);
			batchRecords.setEmployerAddr4(bhFields[23]);
			batchRecords.setEmployerAddr5(bhFields[24]);
			batchRecords.setEmployerState(bhFields[25]);
			batchRecords.setEmployerPin(bhFields[26]);
			batchRecords.setEmployerEmail(bhFields[27]);
			batchRecords.setEmployerStd(bhFields[28]);
			batchRecords.setEmployerPhone(bhFields[29]);
			batchRecords.setEmployerAddrChange(bhFields[30]);
			batchRecords.setDeductorType(bhFields[31]);
			batchRecords.setNameofPersonResponsilbleForSal(bhFields[32]);
			batchRecords.setDesignationofPersonResponsilbleForSal(bhFields[33]);
			batchRecords.setPersonResponsilbleAddr1(bhFields[34]);
			batchRecords.setPersonResponsilbleAddr2(bhFields[35]);
			batchRecords.setPersonResponsilbleAddr3(bhFields[36]);
			batchRecords.setPersonResponsilbleAddr4(bhFields[37]);
			batchRecords.setPersonResponsilbleAddr5(bhFields[38]);
			batchRecords.setPersonResponsilbleState(bhFields[39]);
			batchRecords.setPersonResponsilblePin(bhFields[40]);
			batchRecords.setPersonResponsilbleEmailId1(bhFields[41]);
			batchRecords.setMobileNumber(bhFields[42]);
			batchRecords.setPersonResponsilbleSTDCode(bhFields[43]);
			batchRecords.setPersonResponsilbleTelePhone(bhFields[44]);
			batchRecords.setPersonResponsilbleAddrChange(bhFields[45]);
			batchRecords.setGrossTdsTotalAsPerChallan(bhFields[46]);
			batchRecords.setUnMatchedChalanCnt(bhFields[47]);
			batchRecords.setCountOfSalaryDetailRec(bhFields[48]);
			batchRecords.setGrossTotalIncomeSd(bhFields[49]);
			batchRecords.setApprovalTaken(bhFields[50]);
			batchRecords.setApprovalNo(bhFields[51]);
			batchRecords.setBatchHash(bhFields[52]);
		}
		
		
		return batchRecords;
	}
	
	private static TDSChallanDetailBean getChallanDetailBean(String cdBean[])
	{
		TDSChallanDetailBean cdRecord = new TDSChallanDetailBean();
		cdRecord.setLineNo(cdBean[0]);
		cdRecord.setRecType(cdBean[1]);
		cdRecord.setChBatchNo(cdBean[2]);
		cdRecord.setChallanDetailRecordNo(cdBean[3]);
		cdRecord.setCountOfDeducteeDetail(cdBean[4]);
		cdRecord.setNillChallanIndicator(cdBean[5]);
		cdRecord.setChallanUpdationIndicator(cdBean[6]);
		cdRecord.setPeriod(cdBean[7]);
		cdRecord.setStmtFromDate(cdBean[8]);
		cdRecord.setStmtToDate(cdBean[9]);
		cdRecord.setLastBankChallanNo(cdBean[10]);
		cdRecord.setBankChallanNo(cdBean[11]);
		cdRecord.setLastTransferVoucherNo(cdBean[12]);
		cdRecord.setTransferVoucherNo(cdBean[13]);
		cdRecord.setLastBankBranchCode(cdBean[14]);
		cdRecord.setBankBranchCode(cdBean[15]);
		cdRecord.setLastDateOfBankChallanNo(cdBean[16]);
		cdRecord.setDateOfBankChallanNo(cdBean[17]);
		cdRecord.setMajorHeadCode(cdBean[18]);
		cdRecord.setMinorHeadCode(cdBean[19]);
		cdRecord.setSection(cdBean[20]);
		cdRecord.setOltasIncomeTax(cdBean[21]);
		cdRecord.setOltasSurcharge(cdBean[22]);
		cdRecord.setOltasCess(cdBean[23]);
		cdRecord.setOltasInterest(cdBean[24]);
		cdRecord.setOltasOthers(cdBean[25]);
		cdRecord.setTotalOfDepositAmountAsPerChallan(cdBean[26]);
		cdRecord.setLastTotalOfDepositAmountAsPerChallan(cdBean[27]);
		cdRecord.setTotalTaxDepositedAsPerDeducteeAnex(cdBean[28]);
		cdRecord.setTdsIncomeTaxC(cdBean[29]);
		cdRecord.setTdsSurchargeC(cdBean[30]);
		cdRecord.setTdsCessC(cdBean[31]);
		cdRecord.setSumTotalIncTaxDedAtSource(cdBean[32]);
		cdRecord.setTdsInterest(cdBean[33]);
		cdRecord.setTdsOthers(cdBean[34]);
		cdRecord.setChequeDDNo(cdBean[35]);
		cdRecord.setBookCash(cdBean[36]);
		cdRecord.setRemark(cdBean[37]);
		//Added for FVU 2.134 by Shalin
		cdRecord.setLateFee(cdBean[38]);
		cdRecord.setMinorHeadCodeChaln(cdBean[39]);
		
		cdRecord.setChallanHash(cdBean[40]);
		
		return cdRecord;
	}

	private static TDSDeducteeDetailBean getDeducteeDetailBean(String ddBean[])
	{
		TDSDeducteeDetailBean ddRecord = new TDSDeducteeDetailBean();
		ddRecord.setLineNo(ddBean[0]);
		ddRecord.setRecType(ddBean[1]);
		ddRecord.setDdBatchNo(ddBean[2]);
		ddRecord.setChallanRecordNo(ddBean[3]);
		ddRecord.setDeducteeDetailRecNo(ddBean[4]);
		ddRecord.setMode(ddBean[5]);
		ddRecord.setEmployeeSerialNo(ddBean[6]);
		ddRecord.setDeducteeCode(ddBean[7]);
		ddRecord.setLastEmployeePan(ddBean[8]);
		ddRecord.setEmployeePan(ddBean[9]);
		ddRecord.setLastEmployeePanRefNo(ddBean[10]);
		ddRecord.setEmployeePanRefNo(ddBean[11]);
		ddRecord.setEmployeeName(ddBean[12]);
		ddRecord.setTdsIncomeTaxDD(ddBean[13]);
		ddRecord.setTdsSurchargeDD(ddBean[14]);
		ddRecord.setTdsCessDD(ddBean[15]);
		ddRecord.setTotalIncomeTaxDeductedAtSource(ddBean[16]);
		ddRecord.setLastTotalIncomeTaxDeductedAtSource(ddBean[17]);
		ddRecord.setTotalTaxDeposited(ddBean[18]);
		ddRecord.setLastTotalTaxDeposited(ddBean[19]);
		ddRecord.setTotalValueofPurchase(ddBean[20]);
		ddRecord.setAmountOfPayment(ddBean[21]);
		ddRecord.setDateOnWhichAmountPaid(ddBean[22]);
		ddRecord.setDateOnWhichTaxDeducted(ddBean[23]);
		ddRecord.setDateOfDeposit(ddBean[24]);
		ddRecord.setRateAtWhichTaxDeducted(ddBean[25]);
		ddRecord.setGrossingUpIndicator(ddBean[26]);
		ddRecord.setBookCashEntry(ddBean[27]);
		ddRecord.setDateOfFurnishingTaxDeductionCertificate(ddBean[28]);
		ddRecord.setRemark1(ddBean[29]);
		ddRecord.setRemark2(ddBean[30]);
		ddRecord.setRemark3(ddBean[31]);
		
		//Added for FVU 2.134 by Shalin
		
		ddRecord.setSectionCode(ddBean[32]);
		ddRecord.setCertNumAo(ddBean[33]);
		ddRecord.setTdsRate(ddBean[34]);
		ddRecord.setNatureOfRemittance(ddBean[35]);
		ddRecord.setAckNoForm15CA(ddBean[36]);
		ddRecord.setRemittanceCountry(ddBean[37]);
		
		//ddRecord.setDeducteeHash(ddBean[38]);
		
		// Added For FVU 2.148
		
		ddRecord.setDeducteeEmailId(ddBean[38]);
		ddRecord.setDeducteeContactNo(ddBean[39]);
		ddRecord.setDeducteeAddress(ddBean[40]);
		ddRecord.setTaxIdentificationNo(ddBean[41]);
		ddRecord.setDeducteeHash(ddBean[42]);
		
		// End :: Added For FVU 2.148
		
		return ddRecord;
	}
	
	private static TDSSalaryDetailBean getSalaryDetailBean(String sdBean[])
	{
		TDSSalaryDetailBean sdRecord = new TDSSalaryDetailBean();
		sdRecord.setLineNo(sdBean[0]);
		sdRecord.setRecType(sdBean[1]);
		sdRecord.setDdBatchNo(sdBean[2]);
		sdRecord.setDeducteeDetailRecNo(sdBean[3]);
		sdRecord.setModeSD(sdBean[4]);
		sdRecord.setEmployeeSerialNo(sdBean[5]);
		sdRecord.setEmployeePan(sdBean[6]);
		sdRecord.setEmployeePanRefNo(sdBean[7]);
		sdRecord.setEmployeeName(sdBean[8]);
		sdRecord.setEmployeeCategory(sdBean[9]);
		sdRecord.setPeriodEmploymentFrm(sdBean[10]);
		sdRecord.setPeriodEmploymentTo(sdBean[11]);
		sdRecord.setTotalSalaryExeclud333n334(sdBean[12]);
		sdRecord.setProfitSalSec17(sdBean[13]);
		sdRecord.setCountDeducteeSec16(sdBean[14]);
		sdRecord.setIncomeChargeableUderTheHeadSalaries(sdBean[15]);
		sdRecord.setIncomeLossFrmHouse(sdBean[16]);
		sdRecord.setGrossTotIncome(sdBean[17]);
		sdRecord.setLastGrossTotIncome(sdBean[18]);
		sdRecord.setCountDeducteeChapVIDetails(sdBean[19]);
		sdRecord.setCountSec6A(sdBean[20]);
		sdRecord.setGrossTotDeducteeChapVIDetails(sdBean[21]);
		sdRecord.setTotTaxableIncome340344(sdBean[22]);
		sdRecord.setIncomeTaxOnTotIncome(sdBean[23]);
		sdRecord.setSurcharge(sdBean[24]);
		sdRecord.setCess(sdBean[25]);
		sdRecord.setIncomeTaxRelief(sdBean[26]);
		sdRecord.setNetTaxPayable(sdBean[27]);
		sdRecord.setTotTDSWholeYear(sdBean[28]);
		sdRecord.setShortfallExcessTaxDeduc(sdBean[29]);
		sdRecord.setRemark1(sdBean[30]);
		sdRecord.setRemark2(sdBean[31]);
		sdRecord.setRemark3(sdBean[32]);
		
		//Added for FVU 2.134 by Shalin
		
		sdRecord.setTaxblAmtCurEmp(sdBean[33]);
		sdRecord.setTaxblAmtPreEmp(sdBean[34]);
		sdRecord.setTDSCurEmp(sdBean[35]);
		sdRecord.setTDSPreEmp(sdBean[36]);
		sdRecord.setWhethrTaxDedAtHighrRate(sdBean[37]);
		
		// Added for FVU 5.2 : 2.148
		
		//sdRecord.setSalaryDeducteeHash(sdBean[38]);

		
		sdRecord.setWhethrHouseRentClaimExced(sdBean[38]);
		sdRecord.setCountOfPanLanloard(sdBean[39]);
		sdRecord.setLandlordPan1(sdBean[40]);
		sdRecord.setLandlordName1(sdBean[41]);
		sdRecord.setLandlordPan2(sdBean[42]);
		sdRecord.setLandlordName2(sdBean[43]);
		sdRecord.setLandlordPan3(sdBean[44]);
		sdRecord.setLandlordName3(sdBean[45]);
		sdRecord.setLandlordPan4(sdBean[46]);
		sdRecord.setLandlordName4(sdBean[47]);
		
		sdRecord.setWhethrInterestPaidLender(sdBean[48]);
		sdRecord.setCountOfPanLender(sdBean[49]);
		sdRecord.setLenderPan1(sdBean[50]);
		sdRecord.setLenderName1(sdBean[51]);
		sdRecord.setLenderPan2(sdBean[52]);
		sdRecord.setLenderName2(sdBean[53]);
		sdRecord.setLenderPan3(sdBean[54]);
		sdRecord.setLenderName3(sdBean[55]);
		sdRecord.setLenderPan4(sdBean[56]);
		sdRecord.setLenderName4(sdBean[57]);
		
		sdRecord.setWhethrContribtnPaidByTrusty(sdBean[58]);
		sdRecord.setNameOfSuperannuationFund(sdBean[59]);
		sdRecord.setStartDate(sdBean[60]);
		sdRecord.setEndDate(sdBean[61]);
		sdRecord.setAvgRateOfdeductionOfTaxforLastThreeYears(sdBean[62]);
		sdRecord.setAmountOfRepaidOnAccountOfPrincipalAndInterest(sdBean[63]);
		sdRecord.setAmountOfTaxDeductedOnRepayment(sdBean[64]);
		sdRecord.setGrossTotalIncomeFromSuperannuationFund(sdBean[65]);

		sdRecord.setSalaryDeducteeHash(sdBean[66]);
		
		// End :: added for FVU 5.2 :: 2.148
		return sdRecord;
	}
	
	private static Section16DetailBean getSection16DetailBean(String secBean[])
	{
		Section16DetailBean sec16Record = new Section16DetailBean();
		sec16Record.setLineNo(secBean[0]);
		sec16Record.setRecordType(secBean[1]);
		sec16Record.setBatchNo(secBean[2]);
		sec16Record.setSdRecordNo(secBean[3]);
		sec16Record.setSection16Serial(secBean[4]);
		sec16Record.setSection16ID(secBean[5]);
		sec16Record.setTotalDeduction(secBean[6]);
		sec16Record.setRecordHash(secBean[7]);
		
		return sec16Record;
	}
	
	
	private static SectionVIADetailBean getSectionVIADetailBean(String secBean[])
	{
		SectionVIADetailBean secVIARecord = new SectionVIADetailBean();
		secVIARecord.setLineNo(secBean[0]);
		secVIARecord.setRecordType(secBean[1]);
		secVIARecord.setBatchNo(secBean[2]);
		secVIARecord.setSdRecordNo(secBean[3]);
		secVIARecord.setSectionVIASerial(secBean[4]);
		secVIARecord.setSectionVIAID(secBean[5]);
		secVIARecord.setTotalAmount(secBean[6]);
		secVIARecord.setRecordHash(secBean[7]);
		
		return secVIARecord;
	}
	
	
	//FVU 7.6 BUG FIX new requirement .fvu
	private static TDS194PDetailBean get194PDetailBean(String ninetyFourPBean[])
		{
			TDS194PDetailBean ninetyFourPRecord = new TDS194PDetailBean();
			ninetyFourPRecord.setNetTaxPayable(ninetyFourPBean[26]);

			return ninetyFourPRecord;
		}
	
}
